@@ -9,6 +9,8 @@ from pandapower.control.basic_controller import Controller
 from peext.storage import EnergyStorage
 from overrides import overrides
 
+import numpy as np
+
 NETS_ACCESS = 'nets'
 P_MW = 'p_mw'
 
@@ -241,6 +243,8 @@ class CHPControlMultiEnergy(G2PControlMultiEnergy):
             self.gas_cons = self.ref_gas_consume * self.__load_regulation_multiplier
             self.power_gen = self.gas_cons * self.conversion_factor_kgps_to_mw() * self.efficiency
 
+        self.power_gen = self.power_gen[0] if isinstance(self.power_gen, np.ndarray) else self.power_gen
+
         heat_energy_overall = self.calc_heat(self.gas_cons)
         self.heat_energy = heat_energy_overall * self.heat_regulation 
         self.heat_save_energy = heat_energy_overall * (1 - self.heat_regulation)
