@@ -252,7 +252,7 @@ def is_empty_bus(multinet, bus_id, network_name):
     return len(connected_elements) == 0
 
 def is_empty_junction(multinet, junction_id, network_name):
-    for type in ['sink', 'source']:
+    for type in ['sink', 'source', 'ext_grid']:
         if type in multinet[NETS_ACCESS][network_name]:
             for _, row in multinet[NETS_ACCESS][network_name][type].iterrows():
                 if row['junction'] == junction_id:
