@@ -101,6 +101,9 @@ class P2GNode(RegulatableController):
         """
         return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_load
 
+    def active_power(self) -> float:
+        return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_load
+
     def active_power_capability(self):
         return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_cap
 
@@ -121,6 +124,9 @@ class CHPNode(RegulatableController):
 
     def power_output(self) -> float:
         return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_gen
+
+    def active_power(self) -> float:
+        return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_gen
     
     def storage_load(self):
         return self.network.controller[CONTROLLER_OBJ_KEY][self.id].storage_load
@@ -158,6 +164,9 @@ class P2HNode(RegulatableController):
 
     def power_load(self) -> float:
         return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_load
+
+    def active_power(self) -> float:
+        return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_load
     
     def active_power_capability(self):
         return self.network.controller[CONTROLLER_OBJ_KEY][self.id].power_cap
@@ -465,6 +474,14 @@ class HeatExchangerNode(RegulatableMESModel):
         :rtype: float
         """
         return self._network.heat_exchanger[QEXT_W][self._id]
+
+    def qext_w(self):
+        """Return qext_w
+
+        :return: q cap
+        :rtype: float
+        """
+        return self._network.heat_exchanger[QEXT_W][self._id]
     
     def regulate(self, new_scaling):
         """Set the regulation factor.
@@ -598,8 +615,8 @@ class EmptyJunctionNode(RegulatableMESModel):
         return 1
 
     def mdot_kg_per_s_capability(self):
-        return 0
-
+        return 0    
+    
     def name(self):
         """Return name of the ext grid
 
