@@ -8,8 +8,6 @@ import pandapipes as ppipes
 import pandapipes.multinet as ppm
 import pandapower.networks as pandapowernetworks
 import pandapipes.networks as pipesnetworks
-import pandapipes.plotting as pipeplot
-import pandapower.plotting as powerplot
 import pandapower.control as powercontrol
 
 import pandas as pd
@@ -17,7 +15,7 @@ import numpy as np
 import math
 import random
 
-from peext.controller import CHPControlMultiEnergy, RegulatedG2PControlMultiEnergy, RegulatedP2GControlMultiEnergy
+from peext.controller import CHPControlMultiEnergy, RegulatedG2PControlMultiEnergy, RegulatedP2GControlMultiEnergy, RegulatedP2HControlMultiEnergy
 
 def create_small_test_multinet():
     """Factory for the panda-multinet.
@@ -72,10 +70,10 @@ def create_small_test_multinet():
     ppm.add_net_to_multinet(mn, net_heat, net_name="heat")
 
     # Create coupling point chp    
-    chp_gen = ppower.create_gen(net_power, bus_id3, p_mw = 1, vm_pu = 10, name="CHPTorgenerator")
+    chp_gen = ppower.create_sgen(net_power, bus_id3, p_mw = 1, name="CHPTorgenerator")
     chp_sink = ppipes.create_sink(net_gas, junction=jun_id3, mdot_kg_per_s=0.085, name="CHP Sink")
     chp_heat_feed_in = ppipes.create_heat_exchanger(net_heat, from_junction=jun_id_heat, to_junction=jun_id2_heat, diameter_m=200e-3, qext_w=-10000)
-    CHPControlMultiEnergy(mn, chp_gen, chp_sink, chp_heat_feed_in, 0.8, 300, "CHP1", ambient_temperature=282.5)
+    CHPControlMultiEnergy(mn, chp_gen, chp_sink, chp_heat_feed_in, 0.8, "CHP1", ambient_temperature=282.5)
     
     # Create coupling point power2gas
     p2g_id_el = ppower.create_load(net_power, bus = bus_id2, p_mw = 1, name='ALoad')
@@ -83,9 +81,6 @@ def create_small_test_multinet():
     ppipes.create_source(net_gas, junction = jun_id4, mdot_kg_per_s = 0.2, name='A2Source')
     RegulatedP2GControlMultiEnergy(mn, p2g_id_el, p2g_id_gas, efficiency = 0.8, name="ANp2g")
 
-    #run_timeseries(net_heat, time_steps=range(10), output_writers=create_output_writers(net_heat, 3), mode='all')
-    #plot.simple_plot(net_heat, plot_sinks=True, plot_sources=True)
-    #plot.simple_plot(net_gas, plot_sinks=True, plot_sources=True)
     return mn
 
 def create_big_multinet_gas_power():
@@ -94,8 +89,11 @@ def create_big_multinet_gas_power():
     """
     schutterwald_gas_net = pipesnetworks.schutterwald()
     mv_rhein_power_net = pandapowernetworks.mv_oberrhein()
-    pipeplot.simple_plot(schutterwald_gas_net, plot_sinks=True, plot_sources=True)
-    powerplot.simple_plot(mv_rhein_power_net, plot_loads=True, plot_sgens=True)
+    mn = ppm.create_empty_multinet('multi')
+    ppm.add_net_to_multinet(mn, schutterwald_gas_net, net_name="gas")
+    ppm.add_net_to_multinet(mn, mv_rhein_power_net, net_name="power")
+    return mn
+
 
 def generate_load_profiles(net_gas, net_power, net_heat, time_steps, with_heat, load_type, load_mul):
         # power loads
@@ -269,8 +267,8 @@ def create_medium_multinet_gas_power(load_type=None, load_mul=1, time_steps=101,
         powercontrol.ConstControl(net_heat, element='heat_exchanger', element_index=net_heat.heat_exchanger.index,
                                         variable='qext_w', data_source=ds, profile_name=net_heat.heat_exchanger.index)
             
-        CHPControlMultiEnergy(mn, 0, g_si_1, chp_heat_feed_in, 0.8, 300, "CHP1", ambient_temperature=282.5, element_type_power='sgen')
-        CHPControlMultiEnergy(mn, 1, g_si_2, chp_heat_feed_in, 0.8, 300, "CHP1", ambient_temperature=282.5, element_type_power='sgen')
+        CHPControlMultiEnergy(mn, 0, g_si_1, chp_heat_feed_in, 0.8, "CHP1", ambient_temperature=282.5, element_type_power='sgen')
+        CHPControlMultiEnergy(mn, 1, g_si_2, chp_heat_feed_in, 0.8, "CHP2", ambient_temperature=282.5, element_type_power='sgen')
     else:
         RegulatedG2PControlMultiEnergy(mn, 0, g_si_1, efficiency = 0.9, name="ANg2p2 0 0", element_type_power='sgen')
         RegulatedG2PControlMultiEnergy(mn, 1, g_si_2, efficiency = 0.85, name="ANg2p3 1 1", element_type_power='sgen')
@@ -430,3 +428,147 @@ def create_big_multinet_gas_power(load_type=None, load_mul=1, time_steps=101, wi
     RegulatedP2GControlMultiEnergy(mn, 3, g_s_2, efficiency = 0.94, name="ANp2g2 2 1", element_type_power='gen')
 
     return mn
+
+
+def create_district_gas_network():
+    # Setup L-Gas network
+    REF_BAR = 60
+    REF_TEMP = 290
+    net_gas = ppipes.create_empty_network(name='gas', fluid="lgas")
+    jun_id_ext = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id2 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id3 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id4 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id5 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id6 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id7 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id8 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id9 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+    jun_id10 = ppipes.create_junction(net_gas, pn_bar=REF_BAR, tfluid_k = REF_TEMP)
+
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id6, to_junction=jun_id2, length_km=1.1, diameter_m=0.05, name="Pipe SI 6 SI 2")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id6, to_junction=jun_id5, length_km=2.1, diameter_m=0.05, name="Pipe SI 6 SI 5")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id6, to_junction=jun_id4, length_km=1.1, diameter_m=0.05, name="Pipe SI 6 SI 4")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id5, to_junction=jun_id3, length_km=.5, diameter_m=0.05, name="Pipe SI 5 SI 3")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id2, to_junction=jun_id, length_km=1.1, diameter_m=0.05, name="Pipe SI 2 SI 1")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id7, to_junction=jun_id6, length_km=3.1, diameter_m=0.05, name="Pipe SI 7 SI 6")
+
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id_ext, to_junction=jun_id9, length_km=1.1, diameter_m=0.08, name="Pipe EXT SO 2")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id8, to_junction=jun_id6, length_km=3.1, diameter_m=0.05, name="Pipe SO 1 SI 7")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id9, to_junction=jun_id6, length_km=1.1, diameter_m=0.05, name="Pipe SO 2 SI 6")
+    ppipes.create_pipe_from_parameters(net_gas, from_junction=jun_id10, to_junction=jun_id6, length_km=1.1, diameter_m=0.05, name="Pipe SO 2 SI 6")
+
+    ppipes.create_sink(net_gas, junction=jun_id, mdot_kg_per_s=0.02, name="BSink 1")
+    ppipes.create_sink(net_gas, junction=jun_id2, mdot_kg_per_s=0.045, name="SSink 2")
+    ppipes.create_sink(net_gas, junction=jun_id3, mdot_kg_per_s=0.0085, name="SSink 3")
+    ppipes.create_sink(net_gas, junction=jun_id4, mdot_kg_per_s=0.008, name="MSink 4")
+    ppipes.create_sink(net_gas, junction=jun_id5, mdot_kg_per_s=0.0085, name="SSink 5")
+    ppipes.create_sink(net_gas, junction=jun_id6, mdot_kg_per_s=0.00085, name="SSink 6")
+    ppipes.create_sink(net_gas, junction=jun_id7, mdot_kg_per_s=0.008, name="MSink 7")
+
+    ppipes.create_source(net_gas, junction=jun_id8, mdot_kg_per_s=0.0285, name="BSource 1")
+    ppipes.create_source(net_gas, junction=jun_id9, mdot_kg_per_s=0.0285, name="BSource 2")
+    ppipes.create_source(net_gas, junction=jun_id10, mdot_kg_per_s=0.01885, name="BSource 3")
+    ppipes.create_ext_grid(net_gas, junction=jun_id_ext, p_bar=REF_BAR, t_k=REF_TEMP, name="Grid Connection Gas")
+
+    return net_gas
+
+def create_district_heat_network(number_of_hh_in_sidelanes=20):
+    HEAT_BAR = 5
+    HEAT_TEMP = 363.15
+    net_heat = ppipes.create_empty_network('heat', fluid="water")
+
+    jun_id_heat_main_start = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    jun_id_heat_main_start_2 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    ppipes.create_heat_exchanger(net_heat, from_junction=jun_id_heat_main_start_2, to_junction=jun_id_heat_main_start, diameter_m=0.075, qext_w=-0.1)
+
+    jun_id_heat_main_mid1 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    jun_id_heat_main_mid1_2 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    ppipes.create_heat_exchanger(net_heat, from_junction=jun_id_heat_main_mid1, to_junction=jun_id_heat_main_mid1_2, diameter_m=0.075, qext_w=-0.1)
+    
+    ppipes.create_pipe_from_parameters(net_heat, from_junction=jun_id_heat_main_start, to_junction=jun_id_heat_main_mid1, length_km=4,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)  
+    ppipes.create_pipe_from_parameters(net_heat, from_junction=jun_id_heat_main_mid1_2, to_junction=jun_id_heat_main_start_2, length_km=4,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)  
+
+    jun_id_heat_main_mid2 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    jun_id_heat_main_mid2_2 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    ppipes.create_heat_exchanger(net_heat, from_junction=jun_id_heat_main_mid2_2, to_junction=jun_id_heat_main_mid2, diameter_m=0.075, qext_w=-0.1)
+    
+    ppipes.create_pipe_from_parameters(net_heat, from_junction=jun_id_heat_main_mid1, to_junction=jun_id_heat_main_mid2, length_km=3,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)  
+    ppipes.create_pipe_from_parameters(net_heat, from_junction=jun_id_heat_main_mid2_2, to_junction=jun_id_heat_main_mid1_2, length_km=3,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)  
+
+    jun_id_heat_main_end = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    jun_id_heat_main_end_2 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+    ppipes.create_heat_exchanger(net_heat, from_junction=jun_id_heat_main_end_2, to_junction=jun_id_heat_main_end, diameter_m=0.075, qext_w=-0.1)
+    
+    ppipes.create_pipe_from_parameters(net_heat, from_junction=jun_id_heat_main_mid2, to_junction=jun_id_heat_main_end, length_km=3,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)  
+    ppipes.create_pipe_from_parameters(net_heat, from_junction=jun_id_heat_main_end_2, to_junction=jun_id_heat_main_mid2_2, length_km=3,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)
+
+    for junc_start_point in [(jun_id_heat_main_start, jun_id_heat_main_start_2), 
+              (jun_id_heat_main_mid1, jun_id_heat_main_mid1_2), 
+              (jun_id_heat_main_mid2, jun_id_heat_main_mid2_2),
+              (jun_id_heat_main_end, jun_id_heat_main_end_2)]:
+        connect_1 = junc_start_point[0] 
+        connect_2 = junc_start_point[1]
+        for _ in range(number_of_hh_in_sidelanes):
+            next_jun_1 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+            next_jun_2 = ppipes.create_junction(net_heat, pn_bar=HEAT_BAR, tfluid_k = HEAT_TEMP)
+            ppipes.create_pipe_from_parameters(net_heat, from_junction=connect_1, to_junction=next_jun_1, length_km=3,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)  
+            ppipes.create_pipe_from_parameters(net_heat, from_junction=next_jun_2, to_junction=connect_2, length_km=3,
+                            diameter_m=0.105, k_mm=.025, alpha_w_per_m2k=100, sections = 5, text_k=HEAT_TEMP)  
+            ppipes.create_heat_exchanger(net_heat, from_junction=next_jun_1, to_junction=next_jun_2, diameter_m=0.075, qext_w=-0.1)
+            connect_1 = next_jun_1
+            connect_2 = next_jun_2
+
+    ppipes.create_ext_grid(net_heat, junction=jun_id_heat_main_end, p_bar=HEAT_BAR, t_k=HEAT_TEMP)
+
+    return net_heat
+
+def create_super_district_coupled_network(coupling_density, heat_sidelane_size=20):
+    net_power = pandapowernetworks.create_synthetic_voltage_control_lv_network("village_2")
+    net_power.name = 'power'
+    net_gas = create_district_gas_network()
+    net_heat = create_district_heat_network(heat_sidelane_size)
+
+    # COUPLINGS
+    mn = ppm.create_empty_multinet()
+    ppm.add_net_to_multinet(mn, net_heat, net_name="heat")
+    ppm.add_net_to_multinet(mn, net_power, net_name="power")
+    ppm.add_net_to_multinet(mn, net_gas, net_name="gas")
+
+    # Add coupling points
+    # P2G
+    for i in range(min(len(net_power.load), len(net_gas.source))):
+        if random.random() < coupling_density:
+            RegulatedP2GControlMultiEnergy(mn, int(random.random()*len(net_power.load)), 
+                                               int(random.random()*len(net_gas.source)), 
+                                               efficiency = 0.94,
+                                               name = f"P2G-{i}")
+    # CHP
+    for i in range(min(len(net_power.sgen), len(net_gas.sink), len(net_heat.heat_exchanger))):
+        if random.random() < coupling_density:
+            CHPControlMultiEnergy(mn, int(random.random()*len(net_power.sgen)), 
+                                      int(random.random()*len(net_gas.sink)), 
+                                      int(random.random()*len(net_heat.heat_exchanger)), 
+                                      0.7 + random.random()*0.3,
+                                      "CHP-{i}", 
+                                      ambient_temperature=282.5, 
+                                      element_type_power='sgen')
+
+    # P2H
+    for i in range(min(len(net_power.load), len(net_heat.heat_exchanger))):
+        if random.random() < coupling_density:
+            RegulatedP2HControlMultiEnergy(mn, int(random.random()*len(net_power.load)), 
+                                               int(random.random()*len(net_heat.heat_exchanger)), 
+                                               0.4 + random.random()*0.5,
+                                               name = f"P2H-{i}")
+    
+    return mn
+    
